/*
 * ANSI Console unit (tries to look like CONIO.H)
 * Cursor positions won't match unless only cputch and cputs is used.
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include "ansi.h"

static int conio_x;
static int conio_y;

#define ESC '\x1B'

/* Cursor position: CUP
	esc [#;#H
*/
void gotoxy(int x, int y) {
	printf("\x1B[%d;%dH",y,x);
	conio_x = x;
	conio_y = y;
}

/* send DSR (device status request), receive CPR (cursor position report) */
void deviceStatusReport( void ) {
	char cpr[10] = {0,0,0,0,0,0,0,0,0,0};
	int cprpos=0;
	char in;
	conio_x = -1;
	conio_y = -1;
	fflush(stdin);
	printf("\x1B[6n");
	while ((in=getch())!='\r')
			cpr[cprpos++]=in;
	sscanf(cpr,"\x1B[%d;%dR",&conio_y,&conio_x);
}

int wherex( void ) {
	deviceStatusReport();
	return conio_x;
}

int wherey( void ) {
	deviceStatusReport();
	return conio_y;
}

/* clrscr
	esc [2J
*/
void clrscr( void ) {
	printf("\x1B[2J",ESC);
	gotoxy(1,1);
}


/* clreol
	esc [K
*/
void clreol( void ) {
	printf("\x1B[K",ESC);
}


/* textattr
Set Graphics Rendition (SCR)

	  Esc [#; ... ;#m
Sets the character attribute specified by the parameter(s)
All following characters will have the attribute according
to the parameter(s) util the next occurence of SGR.

Parameter       Meaning
 0            All attributes Off (normal white on black)
 1            Bold On (high intensity)
 4            Underscore On (monochrome mode only)
 5            Blink On
 7            Reverse video On
 8            Cancelled On (invisible)
30            Black foreground
31            Red foreground
32            Green foreground
33            Yellow foreground
34            Blue foreground
35            Magenta foreground
36            Cyan foreground
37            White foreground
40            Black background
41            Red background
42            Green background
44            Yellow background
44            Blue background
45            Magenta background
46            Cyan background
47            White background
*/
void textattr(int ta) {
	unsigned int foregrnd = ta&0x0F;
	unsigned int backgrnd = ta>>4;
	unsigned int fore_int = 0;
	unsigned int back_int = 0;
	unsigned int LUT_foregrnd[] = { 30,34,32,36,31,35,33,37 };
	unsigned int LUT_backgrnd[] = { 40,44,42,46,41,45,43,47 };
	char fg[10];
	char bg[10];

	if (foregrnd>7) {
		foregrnd &= 7;
		fore_int = 1;
	}
	foregrnd = LUT_foregrnd[foregrnd];
	sprintf(fg,"%d;%d",fore_int,foregrnd);


	if (backgrnd>7) {
		backgrnd &= 7;
		back_int = 1;
	}
	backgrnd = LUT_backgrnd[backgrnd];
	if (back_int)
		sprintf(bg,"1;%d",backgrnd);
	else
		sprintf(bg,"%d",backgrnd);

	printf("\x1B[%s;%sm",fg,bg);
}

int cputs( char *p ) {
	conio_x += strlen(p);
	while (conio_x > 80) {
		conio_x -= 80;
		conio_y++;
	}
	if (conio_y>25)
		conio_y = 25;
	printf(p);
	return 0;
}

int putch(int ch) {
	conio_x++;
	if (conio_x>80) {
		conio_x -= 80;
		conio_y++;
	}
	printf("%c",ch);
        return 0;
}



