/*
 * FORMS.H - Header file for forms module.
 *
 *
 */
#ifndef FORMS_H
#define FORMS_H


#include "types.h"
/* Nulstil hukommelsesomrde */
void memclr( char *pnt, long int size );

/* Slet karakter i streng */
void strdel( char *str, int pos );

/* Indst karakter i streng */
void strins( char *str, int pos, char ch );


/*-------------------------------------------------------------------
 * Field beskriver et indtastningsfelt p skrmen. */
struct Field {
	uint tag;			/* brug tag til at markere forskellige felter */
	void *context;		/* context er en pointer til hvad du nsker */
	uint xPos;			/* x-position p skrm */
	uint yPos;			/* y-position p skrm */
	bool readOnly;		/* Dette felt er kun lsning */
	byte promptAttr;  /* Attributter for prompten (forgrund/baggrund) */
	char prompt[80];	/* Prompt p skrm */
	uint maxLen;		/* Maksimal lngde af indtastning */
	uint inputPos;		/* Position i felt */
	uint inputLen;    /* Lngde af felt */
	byte inputAttr;   /* Attributter for feltet */
	char input[80];   /* Buffer til indtastet tekst */
/*	byte flags;          Dette felts flag: 0=alt, 1=0..9, 2=0..9,-,. */
	int (*validator)( struct Field *f );  /* Pointer til den funktion, der validerer feltet. Funktionen returnerer 0 hvis fejlfri, ellers fejlkode. */
};

/* Makro, der gr det nemmere at bygge felterne op */
#define DANFELT(f,tagg,x,y,ro,name,maxlen,defval,validat) \
		  memclr( &(f), sizeof(f) ); \
		  f.tag = tagg; \
		  f.xPos = x; \
		  f.yPos = y; \
		  f.readOnly = ro; \
		  strcpy(f.prompt, name); \
		  f.maxLen = maxlen; \
		  strcpy(f.input, defval ); \
		  f.validator = validat; \
		  f.promptAttr = 0x07; \
		  f.inputAttr = 0x70;

/* Afvikl en form */
void runForm( struct Field fields[], int count );

/* Validators for heltal, floats og Ja/Nej */
int val_int( struct Field *f );
int val_flt( struct Field *f );
int val_yesno( struct Field *f );
/* Validator for kommatal i interval. *context peger p en
 * DoubleLimits struktur udfyldt med passende vrdier */
struct DoubleLimits {
	double min;
	double max;
};
int val_inDoubleRange( struct Field *f );

/* Validator for heltal i interval. *context peger p en
 * IntLimits struktur */
struct IntLimits {
	long int min;
	long int max;
};

int val_inIntRange( struct Field *f );

#endif