/* Graphing.c - this handles all graphics-output
 *
 *
 *
 */

#include <graphics.h>
#include <stdio.h>
#include <math.h>
//#include <conio.h>
#include "ansi.h"

char screenBuffer [4096];
int textx;
int texty;
int maxx;
int maxy;
int GraphDriver;		/* The Graphics device driver		*/
int GraphMode;		/* The Graphics mode value		*/

/* saveText() - saves textscreen before entering graphicsmode */
void saveText(int *x, int *y, char *buffer)
{
	*x = wherex(); /* get current text-cursor position */
	*y = wherey(); /* get current text-cursor position */
	gettext(1,1,80,25, buffer);
}


/* restoreText() - restores textscreen when leaving graphicsmode */
void restoreText(int x, int y, char *buffer)
{
	puttext(1,1,80,25, buffer);
	gotoxy(x, y);
}


/* g_init() - grahpics-mode initialisation */
int g_init()
{
	saveText(&textx, &texty, screenBuffer);
	GraphDriver = DETECT; 			/* Request auto-detection	*/
	initgraph( &GraphDriver, &GraphMode, "" );
	if (graphresult() != 0 )
	{
		restoreText(textx, texty, screenBuffer);
		return 0;
	}
	maxx = getmaxx();
	maxy = getmaxy();
	return 1;
}

/* Leave graphics mode */
void g_exit()
{
	closegraph(); 					/* end graphics-mode */
	restoreText(textx, texty, screenBuffer);
}



/* ViewPort system for abstraction from real graphic resolution
 * into virtual coordinates counting from
 * minX < x < maxX and minY < y < maxY
 */
struct Diagram {
	int x, y; /* Position on screen */
	int w, h; /* Size on screen */
	int posX, posY; /* Aktuel position i Diagram */

	double scaleX, scaleY; /* Skaleringsfaktor p skrm */
	double minX, maxX, minY, maxY; /* Virtuel strrelse */
};

void vp_LineTo(struct Diagram *d, double x, double y ) {
	int newY = (d->y+d->h) - floor((y-d->minY)*d->scaleY);
	int newX = floor( (x - d->minX)*d->scaleX) + d->x;

	line(d->posX, d->posY, newX, newY);
	d->posX = newX;
	d->posY = newY;
}

void vp_MoveTo(struct Diagram *d, double x, double y ) {
	d->posY = (d->y+d->h) - floor((y-d->minY)*d->scaleY);
	d->posX = floor((x+abs(d->minX))*d->scaleX) + d->x;
}

void vp_Init(struct Diagram *d, int sx1, int sy1, int sx2, int sy2) {
	d->x = sx1;
	d->w = sx2-sx1;
	d->scaleX = d->w/(d->maxX - d->minX);
	d->y = sy1;
	d->h = sy2-sy1;
	d->scaleY = d->h/(d->maxY - d->minY);
	d->posX = d->x;
	d->posY = d->y;
/*	rectangle(sx1, sy1, sx2, sy2); */
	vp_MoveTo(d, d->minX, 0.0);
	vp_LineTo(d, d->maxX, 0.0);
	vp_MoveTo(d, 0.0, d->minY);
	vp_LineTo(d, 0.0, d->maxY);
}

/* Method for drawing labels on screen */
void labelOut(int x, int y, char *label) {
	moveto(x,y);
	outtext(label);
}

/* extended version for label printing */
void labelOutF ( int x, int y, double d, char *s)
{
	char buffer[40];
	sprintf(buffer, "%s%6.3lg", s, d);
	labelOut(x,y,buffer);
}

/* drawScreen() - draws the background, title, etc. */
void drawScreen()
{
	char msg[] = "RCL-Simulator - plot";
	int len = textwidth(msg);


	setbkcolor(BLUE);
	/* draw frame */
/*	rectangle(0,0,maxx, maxy); */
	moveto(0, 10);
	lineto(maxx, 10);
	labelOut((maxx-len)/2,2,msg);
}

void Pause()
{
  getch();				/* Read a character from kbd	*/
}




int plotTable(struct SimEntry *res, struct SimParam *s )
{
	struct Diagram tu, ti;
	int lp1;
	struct SimEntry *looper;
	double maxA=0.0, minA=0.0, maxV=0.0, minV=0.0;
	/* Find max og min vrdier i resultat-tabeller */
	looper = res;
	for (lp1=0; lp1<s->points; looper++, lp1++) {
		maxV = MAX(looper->volt, maxV);
		minV = MIN(looper->volt, minV);
		maxA = MAX(looper->mAmp, maxA);
		minA = MIN(looper->mAmp, minA);
	}

	if (g_init() == 0) return 0;	/* save textscreen and initialize graphics, return if error */

	drawScreen();
	labelOutF(10,25, s->L*1e3,  "L(mH)= ");
	labelOutF(10,35, s->C*1e6,  "C(uF)= ");
	labelOutF(10,45, s->R*1e-3,  "R(kO)= ");
	labelOutF(10,55, s->U0, "U0(V)= ");
	labelOutF(10,65, s->T*1e3,  "T(ms)= ");
	labelOutF(10,75, s->points, "N    = ");
	labelOut (maxx-140, 25, "Spnding");
	labelOutF(maxx-140, 35, maxV, "Vmax(V)= ");
	labelOutF(maxx-140, 45, minV, "Vmin(V)= ");
	labelOut (maxx-140, maxy/2+5, "Strm");
	labelOutF(maxx-140, maxy/2+15, maxA, "Imax(mA)= ");
	labelOutF(maxx-140, maxy/2+25, minA, "Imin(mA)= ");


	tu.maxX = s->T*1e3;
	tu.minX = 0.0;
	tu.maxY = ceil(maxV);
	tu.minY = floor(minV);
	vp_Init(&tu, 120,20,maxx-10,maxy/2 );
	ti.maxX = s->T*1e3;
	ti.minX = 0.0;
	ti.maxY = ceil(maxA);
	ti.minY = floor(minA);
	vp_Init(&ti, 120,maxy/2+10,maxx-10,maxy-10 );

	looper = res;
	for (lp1=0; lp1<s->points; looper++, lp1++)
	{
		vp_LineTo( &tu, looper->mSec, looper->volt);
		vp_LineTo( &ti, looper->mSec, looper->mAmp);
	}





	Pause();
	g_exit(); /* close grahicsmode and restore textscreen */

	return 1;
}
