/* RCL simuleringsprogram
 *
 * Bruger:
 *   TYPES.H - alle globale typer er defineret i dette modul
 *   FORMS.C - modul til skrmbilled-styring
 *   RCLMAIN.C - kildekode til hovedprogram
 *
 *
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <math.h>
#include <string.h>
/* #include <conio.h> */

#include "ansi.h"
#include "forms.h"
#include "types.h"
#include "error.h"

/* #include "graphing.c" */

bool isYesNo( char *s )
{
	switch (*s) {
		case 'j': case 'J': return TRUE;
		case 'n': case 'N': return FALSE;
		default: return FALSE;
	}
}



/* Tester om filen eksisterer, dog ignoreres ""
 */
int existsval( struct Field *f ) {
	if (f->inputLen<=0)
		return E_SUCCESS;

	if (access(f->input,0)==0)
		return E_SUCCESS;
	else
		return E_FILE_MUST_EXIST;
}





/* Denne function returnerer en pointer til en tabel af SimEntry's */
struct SimEntry *simulate( struct SimParam *params )
{
	int step=0;

	struct SimEntry *result = (struct SimEntry*)malloc( sizeof(struct SimEntry)*params->points );

	struct SimEntry *cur = result; /* pointer til det element der skal udfyldes */

	if (result==NULL) return NULL;

	params->frq = sqrt(1/(params->L*params->C));
	params->damp = (1.0/2.0) * params->R * sqrt(params->C/params->L);

	cur->mSec = 0.0;
	cur->volt = 0.0;
	cur->mAmp = 0.0;
	cur++;

	for (step=1; step<params->points; step++, cur++)
	{
		float simd = sqrt(1-params->damp*params->damp);
		float cTime = step*params->T/params->points;
		float ee = (1/simd)*exp(-params->damp*params->frq*cTime);

		cur->mSec = cTime * 1000;
		cur->volt = params->U0 * ( 1-ee*sin(simd*params->frq*cTime-atan(-simd/params->damp)));
		cur->mAmp = (params->U0 / (params->L * params->frq)) * ee * sin(simd*params->frq*cTime) * 1000;

	}

	return result;
}

/* Udskriver simulationsresultatet fra y'd linje og ned
 * s peger p en tabel af SimEntry's
 */
void printTable( int y, struct SimEntry *s, int count )
{
	int lp1;
	int rowcount = count/3+(count%3>0?1:0); /* antal i hver kolonne */


	gotoxy(1,y);
	textattr(0x07);

	printf(" mSek    Volt    mAmp      mSek    Volt    mAmp      mSek    Volt    mAmp\n");
	for (lp1=0; lp1<rowcount; lp1++) {
		printf("%6.3f  %6.3f  %6.3f    ",(s+lp1)->mSec, (s+lp1)->volt, (s+lp1)->mAmp);
		if (lp1+rowcount < count)
			printf("%6.3f  %6.3f  %6.3f    ",(s+lp1+rowcount)->mSec, (s+lp1+rowcount)->volt, (s+lp1+rowcount)->mAmp);
		if (lp1+rowcount*2 < count)
			printf("%6.3f  %6.3f  %6.3f",(s+lp1+2*rowcount)->mSec, (s+lp1+2*rowcount)->volt, (s+lp1+2*rowcount)->mAmp);
		printf("\n");
	}

}



#define MAGIC 0x0736C6372L


/* Indlser resultater fra en fil. Der sendes en pointer til en record,
 * der fyldes med simulationsparametrene og der returneres en pointer
 * til en tabel fyldt med simulationsresultaterne.
 * Der returneres NULL hvis der opstr en fejl under lsning.
 */
struct SimEntry *readFromFile( char *filename, struct SimParam *params )
{
	FILE *f;
	struct SimEntry *read;
	ulong magic = 0;
	char fname[255], *p;

	strcpy(fname, filename);
	p = strrchr(fname,'.');
	if (p==NULL)
		strcat(fname,".DAT");

	f = fopen( fname, "rb" );
	if (f==NULL) {
		errormsg(E_FILE_OPEN_ERR);
		return NULL;
	}

	if (fread( &magic, sizeof(magic), 1, f)!=1 || magic!=MAGIC) {
		errormsg(E_FILE_READ_ERR);
		fclose(f);
		return NULL;
	}

	if (fread( params, sizeof(*params), 1, f)!=1) {
		errormsg(E_FILE_READ_ERR);
		fclose(f);
		return NULL;
	}

	read = (struct SimEntry*)malloc( sizeof(struct SimEntry)*params->points );

	if (fread( read, sizeof(struct SimEntry), params->points, f)!=params->points) {
		errormsg(E_FILE_READ_ERR);
		free(read);
		fclose(f);
		return NULL;
	}

	if (fclose(f)) {
		errormsg(E_FILE_CLOSE_ERR);
		return NULL;
	}

	return read;
}



/* Resultatet af en simulation udskrives til en binr fil.
 * I params leveres en pointer til en record, der beskriver
 * simulationsparametrene og i res leveres pointeren til tabellen
 * af simulationsresultater.
 * Der returneres TRUE hvis filen vellykket skrives, ellers FALSE.
 * filename fr tilfjet .DAT hvis der ikke er et efternavn i forvejen
 */
bool writeToFile( char *filename, struct SimParam *params, struct SimEntry *res )
{
	FILE *f;
	struct Field overwrite;
	ulong magic = MAGIC;
	char *fname=filename, *p;

	p = strrchr(fname,'.');
	if (p==NULL)
		strcat(fname,".DAT");



	if (access(fname,0)==0) {
		p = (char *)malloc(40); memclr(p,40);
		sprintf(p,"Overskriv filen '%s'? ",fname);
		DANFELT(overwrite,0,10,24,FALSE,p,1,"",&val_yesno);
		free(p);
		runForm(&overwrite,1);
		textattr(0x07); gotoxy(1,24); clreol();
		if (!isYesNo(overwrite.input))
			return FALSE;
	}

	f = fopen( fname, "wb" );
	if (f==NULL) {
		errormsg(E_FILE_OPEN_ERR);
		return FALSE;
	}

	if (fwrite( &magic, sizeof(magic), 1, f)!=1) {
		errormsg(E_FILE_WRITE_ERR);
		fclose(f);
		remove(filename);
		return FALSE;
	}

	if (fwrite( params, sizeof(*params), 1, f)!=1) {
		errormsg(E_FILE_WRITE_ERR);
		fclose(f);
		remove(filename);
		return FALSE;
	}


	if (fwrite( res, sizeof(struct SimEntry), params->points, f)!=params->points) {
		errormsg(E_FILE_WRITE_ERR);
		fclose(f);
		remove(filename);
		return FALSE;
	}

	if (fclose(f)) {
		errormsg(E_FILE_CLOSE_ERR);
		return FALSE;
	}


	return TRUE;
}






int main( void ) {
	int lp1;
	struct Field filPrompt;
	struct Field afslut;
	struct Field plotSvar;
	struct Field f[10];
	struct SimParam s;
	struct SimEntry *res = NULL;
	struct DoubleLimits R_range={0.0,1.5}, T_range={1.0,10.0}, U_range={1.0,10.0};
	struct IntLimits N_range={5,45};

	do {

		/* Initialisering af indtastningsfelter */
		DANFELT(f[0],0, 1,3,TRUE ,"L( mH ) = ",10,"125",NULL);
		DANFELT(f[1],1, 1,4,TRUE ,"C( uF ) = ",10,"0.2",NULL);
		DANFELT(f[2],2, 1,5,FALSE,"R(kOhm) = ",10,".3",&val_inDoubleRange); f[2].context = &R_range;
		DANFELT(f[3],3,25,3,FALSE,"U( V  ) = ",10,"5",&val_inDoubleRange); f[3].context = &U_range;
		DANFELT(f[4],4,25,4,FALSE,"T(mSek) = ",10,"2",&val_inDoubleRange); f[4].context = &T_range;
		DANFELT(f[5],5,25,5,FALSE,"N       = ",10,"45",&val_inIntRange); f[5].context = &N_range;
		DANFELT(f[6],6,50,3,FALSE,"ID      = ",MAX_USER_NAME,"HANS",NULL);
		DANFELT(f[7],7,50,4,TRUE ,"DAMP    = ",10,"",NULL);
		DANFELT(f[8],8,50,5,TRUE ,"FRQ(s-1)= ",10,"",NULL);
		DANFELT(f[9],9,62,1,TRUE ,"Aktivitet: ",8,"interakt",NULL);
		f[9].inputAttr = 0x87;
		DANFELT(plotSvar,99,5,24,FALSE,"Plot tidssvar? (J/N) ",1,"",&val_yesno);
		DANFELT(filPrompt,98,5,24,FALSE,"",32,"",existsval);
		DANFELT(afslut,100,5,24,FALSE,"Afslut programmet? (J/N) ",1,"",&val_yesno);

		textattr(0x07);
		clrscr();
		gotoxy(1,1); textattr(0x0F); cputs("RCL-SIMULATOR"); textattr(0x07); cputs(" - by HomeCooked Softwarez");
		gotoxy(1,2); for (lp1=0;lp1<80;lp1++) putch('');
		runForm(&f[9],1);

		lp1 = FALSE;
		do {
			strcpy(filPrompt.prompt, "Indls fra fil (blank for ingen): ");
			runForm( &filPrompt, 1 );
			textattr(0x07); gotoxy(1,24); clreol();

			if (filPrompt.inputLen>0) {
				res = readFromFile( filPrompt.input, &s );
				if (res == NULL)
					lp1 = TRUE;
			} else lp1 = FALSE;
		} while (lp1);




		/* Hvis der ikke indlses noget krer vi hele showet med indtastningsfelter */
		if (res==NULL) {
			runForm( f, 10 );
			strcpy(f[9].input,"BEREGNER");
			sscanf(f[0].input,"%lf",&s.L); s.L *=1e-3;
			sscanf(f[1].input,"%lf",&s.C); s.C *=1e-6;
			sscanf(f[2].input,"%lf",&s.R); s.R *=1e3;
			sscanf(f[3].input,"%lf",&s.U0);
			sscanf(f[4].input,"%lf",&s.T); s.T *=1e-3;
			sscanf(f[5].input,"%d",&s.points);
			strcpy(s.user,f[6].input);
		}
		/* Her gentegner vi alle felter med deres bestemte vrdier */
		for (lp1=0; lp1<10; lp1++)
			f[lp1].readOnly = TRUE;
		sprintf(f[0].input,"%-8.4g",s.L*1e3);
		sprintf(f[1].input,"%-8.6g",s.C*1e6);
		sprintf(f[2].input,"%-8.6g",s.R*1e-3);
		sprintf(f[3].input,"%-8.6g",s.U0   );
		sprintf(f[4].input,"%-8.6g",s.T*1e3);
		sprintf(f[5].input,"%d",s.points);
		strcpy(f[6].input,s.user);
		runForm( f, 10 );
		if (res==NULL)
			res = simulate( &s );



		printTable( 7, res, s.points );
		sprintf(f[7].input,"%-8.6f",s.damp);
		sprintf(f[8].input,"%-8.6f",s.frq);


		strcpy(f[9].input,"interakt");
		runForm( f, 10);



		runForm(&plotSvar,1);
		textattr(0x07); gotoxy(1,24); clreol();

/*                if (isYesNo(plotSvar.input)) {
			if (plotTable( res, &s ) == 0) errormsg(E_GRAPHICS_ERROR);
		}
*/


		do {
			strcpy(filPrompt.prompt,"Skriv til fil (blank hvis ingen): ");
			filPrompt.validator = NULL;
			runForm(&filPrompt, 1);
			textattr(0x07); gotoxy(1,24); clreol();

			if (strlen(filPrompt.input) > 0) {
				lp1 = writeToFile( filPrompt.input, &s, res );
				gotoxy(5,24); printf("Skrev filen '%s'",filPrompt.input); getch();
				gotoxy(1,24); clreol();
			} else
				lp1 = TRUE;
		} while (!lp1);


		/* Sprg om programmet skal afsluttes */
		runForm(&afslut,1);
		free( res );
		res = NULL;

	} while (!isYesNo(afslut.input));

	textattr(0x07);
	clrscr();
	printf("Thanx for using HomeCooked softwarez RCL-SIMULATOR.\nThis product is freeware - source code is distrubuted under our lack of control.\nPlease come back soon...\n");

	return 0;
}

