/* FORMS.C - this defines the form's system, where a form must be filled.
 *
 * Nr man bruger FORMS systemet laver man en tabel af strukturer af
 * typen "Field". Disse udfyldes (anvend DANFELT makroen). Derefter
 * kaldes runForm(). runForm vil optegne alle felter p skrmen og finde
 * det frste felt, der ikke er read-only. Hvis alle felter er read-only
 * returneres efter optegning. runForm returnerer nr alle felter er
 * udfyldt og valideret og brugeren trykker p <enter>.
 *
 * HUSK - Der kan kun laves felter p 80 karakterer med en prompt p 80
 *        karakterer.
 *
 * HUSK - det er de strukturer, der sendes, der udfyldes!
 *
 * HUSK - en "validator" har formatet "int val( struct Field *f )" og
 *        returnerer 0 hvis validering OK, ellers en fejlkode til
 *        errormsg().
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
/* #include <conio.h> */
#include "ansi.h"
#include "types.h"
#include "forms.h"
#include "error.h"



/*-------------------------------------------------------------------
 * En lille hjlperutine til at stte et hukommelsesomrde
 * til lutter 0'er.
 */
void memclr( char *pnt, long int size ) {
	int lp1=0;
	for ( ; lp1<size; lp1++, pnt++)
		*pnt = 0;
}


/*-------------------------------------------------------------------
 * Sletter karakteren p pos i strengen str
 */
void strdel( char *str, int pos )
{
	int lp1;
	int len = strlen(str);

	if (pos>=len) return;

	str +=pos;
	for (lp1=pos; lp1<len; lp1++, str++)
		*str = *(str+1);
	*(str+1) = 0;
}


/* Indstter karakteren ch p position pos i str
 */
void strins( char *str, int pos, char ch )
{
	char *s = str;
	int lp1;
	int len = strlen(str);
	if (pos>len) return;

	if (pos==len) {
		*(str+len) = ch;
		*(str+len+1) = 0;
		return;
	}

	s+=len;
	for (lp1=len; lp1>=pos; lp1--, s--)
		*(s+1) = *s;
	*(s+1) = ch;
}






/*-------------------------------------------------------------------
 * Her defineres de tastekoder, der anvendes i BIOS */
#define KC_LEFT  75
#define KC_RIGHT 77
#define KC_UP    72
#define KC_DOWN  80
#define KC_INS   82
#define KC_HOME  71
#define KC_END   79
#define KC_DEL   83
#define KC_ESC   27
#define KC_SHIFT_TAB 15
#define KC_TAB   '\t'
#define KC_BACKSPACE '\b'
#define KC_RETURN '\r'
#define KC_ENTER '\r'

/* Hvert felt kan returnere flgende resultat af editeringen */
enum KeyPress { kpNone, kpDone,kpNext, kpPrev, kpEsc };

/* Her gennemfres selve indtastningsdelen af feltet. "in" peger
 * p det felt, der behandles
 */
enum KeyPress processField( struct Field *in )
{
	static bool insert;
	enum KeyPress retval = kpNone;
	bool redraw;      /* Skal feltet genoptegnes */
	unsigned char ch;
	int er;
	int lp1 = 0;
	int fieldstart = in->xPos + strlen(in->prompt);
	in->inputLen = strlen(in->input);
	in->inputPos = MIN(in->inputPos, in->inputLen);



	/* Write field to screen */
	textattr(in->promptAttr);
	gotoxy(in->xPos, in->yPos);
	cputs(in->prompt);
	textattr(in->inputAttr);
	gotoxy(fieldstart, in->yPos);
	for (lp1=0; lp1<in->maxLen; lp1++)
		putch(lp1>in->inputLen?' ':in->input[lp1]);
	gotoxy(fieldstart+in->inputPos, in->yPos);

	/* Enter keyprocessing */
	if (in->readOnly) retval = kpNext;
	while (retval == kpNone) {

		ch = getch();

		redraw = FALSE;

		/* process extended keys (first char is 0) */
		if (ch==0) switch (ch=getch()) {

			case KC_LEFT:
				if (in->inputPos>0) in->inputPos--;
				else errormsg(E_ILLEGAL_EDIT);
				break;

			case KC_RIGHT:
				if (in->inputPos+1<=in->inputLen) in->inputPos++;
				else errormsg(E_ILLEGAL_EDIT);
				break;

			case KC_UP: case KC_SHIFT_TAB:
				if (in->validator==NULL)
					retval = kpPrev;
				else
					if (!(er=(*in->validator)(in)))
						retval = kpPrev;
					else errormsg(er); break;

			case KC_DOWN:
				if (in->validator==NULL)
					retval = kpNext;
				else
					if (!(er=(*in->validator)(in)))
						retval = kpNext;
					else errormsg(er); break;

			case KC_HOME:
				in->inputPos=0;
				break;

			case KC_END:
				in->inputPos=in->inputLen;
				break;

			case KC_INS:
				insert = !insert;
				break;

			case KC_DEL:
				if (in->inputLen>0 && in->inputPos<in->inputLen) {
					strdel(in->input, in->inputPos);
					in->inputLen--;
					redraw = TRUE;
				} else
					errormsg(E_ILLEGAL_EDIT);
				break;

		} else

		/* Ordinary edit key */
		if (ch<32) switch (ch) {
			case KC_BACKSPACE:
				if (in->inputLen>0 && in->inputPos>0) {
					in->inputPos--;
					in->inputLen--;
					strdel(in->input, in->inputPos);
					redraw = TRUE;
				} else
					errormsg(E_ILLEGAL_EDIT);
				break;

			case KC_RETURN:
				if (in->validator==NULL)
					retval = kpDone;
				else
					if (!(er=(*in->validator)(in)))
						retval = kpDone;
					else errormsg(er);
				break;

			case KC_TAB:
				if (in->validator==NULL)
					retval = kpNext;
				else
					if (!(er=(*in->validator)(in)))
						retval = kpNext;
					else errormsg(er);
				break;

			case KC_ESC:
				if (in->validator==NULL)
					retval = kpEsc;
				else
					if (!(er=(*in->validator)(in)))
					retval = kpEsc;
				break;

		} else

		/* entry keys */
			if (in->inputLen<in->maxLen) {
				strins(in->input,in->inputPos,ch);
				in->inputPos++;
				in->inputLen++;
				redraw = TRUE;
			} else
				errormsg(E_TOO_MANY_CHARS);


		/* Hvis der er ndret noget i teksten, da genoptegnes feltet helt */
		if (redraw || retval!=kpNone) {
			textattr(in->inputAttr);
			gotoxy(fieldstart, in->yPos);
			for (lp1=0; lp1<in->maxLen; lp1++)
				putch(lp1>in->inputLen?' ':in->input[lp1]);
		}
		gotoxy(fieldstart+in->inputPos, in->yPos);

	} /*while*/


	return retval;
}


/* Her afvikles en hel tabel af felter. "fields" er (en pointer til)
 * en tabel fuld af struct's, som er de felter, der skal behandles p
 * skrmen. Der kan skiftes mellem de enkelte felter med pil-op, pil-ned,
 * og hele indtastningen afsluttes med enter.
 * Fr indtastningen afsluttes testes alle felter med validate-funktionen,
 * s kun nr alle felter godkendes kan runForm afslutte.
 */
void runForm( struct Field fields[], int count )
{
	int curField = 0;
	int newField = 0;
	bool stop = FALSE;
	enum KeyPress kp;
	int er;

	/* Tegn alle felter p skrm */
	for (newField=0; newField<count; newField++) {
		gotoxy(fields[newField].xPos, fields[newField].yPos);
		textattr(fields[newField].promptAttr);
		cputs(fields[newField].prompt);
		textattr(fields[newField].inputAttr);
		fields[newField].inputLen = strlen(fields[newField].input);
		for (er=0; er<fields[newField].maxLen; er++)
			putch(er>fields[newField].inputLen?' ':fields[newField].input[er]);
	}

	/* Find det frste indtastningsfelt */
	for (curField=0; curField<count; curField++)
		if (!fields[curField].readOnly) break;
	if (curField>=count || fields[curField].readOnly)
		return ;

	stop = FALSE;
	do {
		kp=processField( &fields[curField] );
		switch (kp) {
			case kpNext:
				curField = (curField+1)%count;
				break;
			case kpPrev:
				newField = (curField<1?count-1:curField-1);
				while (fields[newField].readOnly && newField!=curField) newField = (newField<1?count-1:newField-1);
				curField = newField;
				break;
			case kpDone:
				stop = TRUE;
				for (newField=0; newField<count; newField++)
					if (fields[newField].validator!=NULL)
						if (0<(er=(*fields[newField].validator)(&fields[newField]))) {
						curField = newField;
						gotoxy(fields[curField].xPos+strlen(fields[curField].prompt)+fields[curField].inputPos,fields[curField].yPos);
						errormsg(er);
						stop = FALSE;
						newField = count;
					}
				break;
		}
	} while (!stop);


}




/* Et eksempel p en validator, der undersger om der er indtastet et
 * heltal
 */
int val_int( struct Field *f ) {
	long int i = 0;
	if (sscanf( f->input, "%d", &i)==1) {
/*		sprintf(f->input,"%d",i);
		f->inputLen = strlen(f->input);*/
		return E_SUCCESS;
	}
	else
		return E_MUST_BE_INT;
}



/* Endnu et eksempel p en validator, der kun accepterer floats
 */
int val_flt( struct Field *f ) {
	float i = 0.0;
	if (sscanf( f->input, "%f", &i)==1) {
/*		sprintf(f->input, "%.2f", i);
		f->inputLen = strlen(f->input);*/
		return E_SUCCESS;
	}
	else
		return E_MUST_BE_FLOAT;
}


/* Denne validator accepterer kun enten J eller N (frste kar)
 */
int val_yesno( struct Field *f ) {
	switch (f->input[0]) {
		case 'J': case 'j':
		case 'N': case 'n':
			return E_SUCCESS;
		default:
			return E_MUST_BE_YESNO;
	}
}


/* Et eksempel p en validator, der undersger om der er indtastet et
 * kommatal i det interval, der er angivet i strukturen peget til
 * af context.
 */
int val_inDoubleRange( struct Field *f ) {
	double d = 0.0;
	if (sscanf( f->input, "%lf", &d)==1) {
		if (f->context!=NULL && (
		    d<(*(struct DoubleLimits*)(f->context)).min ||
		    d>(*(struct DoubleLimits*)(f->context)).max) ) {
			sprintf(lasterror,errorstr[E_MUST_BE_IN_RANGE],(*(struct DoubleLimits*)(f->context)).min,(*(struct DoubleLimits*)(f->context)).max);
			return E_LASTERROR;
		}
		else return E_SUCCESS;
	}
	else
		return E_MUST_BE_FLOAT;
}

/* Et eksempel p en validator, der undersger om der er indtastet et
 * heltal i et interval
 */
int val_inIntRange( struct Field *f ) {
	long int i = 0;
	if (sscanf( f->input, "%d", &i)==1) {
		if (f->context!=NULL && (
		    i<(*(struct IntLimits*)(f->context)).min ||
		    i>(*(struct IntLimits*)(f->context)).max) ) {
			sprintf(lasterror,errorstr[E_MUST_BE_IN_INT_RANGE],(*(struct IntLimits*)(f->context)).min,(*(struct IntLimits*)(f->context)).max);
			return E_LASTERROR;
		}
		else return E_SUCCESS;
	}
	else
		return E_MUST_BE_INT;
}
